#include "Affichage.hpp"

#include <SDL_framerate.c>

// Constructeur
Affichage::Affichage ()
	: _debug (true)
{
	// On rcupre l'instance vers le singleton NewtWorld
	_nWorld = NewtWorld::GetInstance ();

	_vBody.reserve (10); // On reserve de la place pour 10 lments
}

// Destructeur
Affichage::~Affichage ()
{
	Shutdown ();
}

// Initialisation de la SDL
void Affichage::InitSDL ()
{
	// Initialisation de SDL
   SDL_Init (SDL_INIT_VIDEO);

   SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1); // On active le double buffering
	SDL_GL_SetAttribute (SDL_GL_MULTISAMPLESAMPLES, 4); // On active un lger anti-aliasing, juste
																		 // pour le rendu. Si votre PC ne suit pas,
																		 // commentez cette ligne.

   SDL_Surface * pEcran = SDL_SetVideoMode (800, 600, 32, SDL_OPENGL);

   if (pEcran == NULL)
   {
      std::cerr << "Erreur au paramtrage vido : " << SDL_GetError ();
   }

   atexit (SDL_Quit);
}

// Initialisation d'OpenGL
void Affichage::InitOGL ()
{
	glClearColor(0.0f, 0.0f, 0.3f, 0.0f);
   glClearDepth(1.0);

   GLenum err = glewInit();

   if (GLEW_OK != err)
      std::cerr << "Erreur dans GLEW.";

   glViewport (0, 0, 800, 600); // On rgle le Viewport

   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluPerspective (60.0, 800.0 / 600.0, 0.1, 1000.0);

   glMatrixMode (GL_MODELVIEW);
   glLoadIdentity (); // On rtablit la matrice identit

   glEnable (GL_DEPTH_TEST);
   glEnable (GL_COLOR_MATERIAL);
   glEnable (GL_LIGHTING);

   glEnable (GL_LIGHT0);
   GLfloat LampeDiffuse [] = {1.0f, 1.0f, 1.0f};
   GLfloat LampeAmbient [] = {0.75f, 0.75f, 0.75f};
   GLfloat LampePosition [] = {50.0f, 50.0f, 0.0f, 0.0f};

   glLightfv (GL_LIGHT0, GL_DIFFUSE, LampeDiffuse);
   glLightfv (GL_LIGHT0, GL_SPECULAR, LampeDiffuse);
   glLightfv (GL_LIGHT0, GL_AMBIENT, LampeAmbient);
   glLightfv (GL_LIGHT0, GL_POSITION, LampePosition);
}

// Boucle principale du programme
void Affichage::Boucle ()
{
	SDL_Event event;
	bool bQuit = false;

	FPSmanager manager;
	SDL_initFramerate (&manager);
	SDL_setFramerate (&manager, 80);

	while (bQuit == false)
	{
		SDL_framerateDelay (&manager);
		
		GetFPS ();
      Update ();
		Render ();

		if (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_QUIT:
               Shutdown ();
               bQuit = true;
               break;
            case SDL_KEYDOWN:
               switch (event.key.keysym.sym)
               {
                  case SDLK_ESCAPE:
                     Shutdown ();
                     bQuit = true;
                     break;      
						case SDLK_F1:
							_debug = !_debug;
							break;
                  default:
                     break;
               }
            default:
               break;
			}
		}
	}
}

void Affichage::GetFPS ()
{
   static char titleBar[50] = {0};
   static int FPS = 0;
   float nextSecond = 0.0f;
   static float prevSecond = 0.0f;

   FPS++;

   nextSecond = SDL_GetTicks() * 0.001f;

   if (nextSecond - prevSecond > 1.0f)
   {
         prevSecond = nextSecond;

			sprintf(titleBar, "%d FPS - F1 : Activation/Desactivation mode debug", FPS);

         SDL_WM_SetCaption (titleBar, NULL);

         FPS = 0;
   }
}

// Charge les lments de la scne
void Affichage::InitScene ()
{	
	// On tourne  80 fps.
	_nWorld->NewtSetMinimumFramerate (80);

	/// SOL
	CModel * sol = new CModel();
	sol->OuvrirFichier ("sol.ase");

	/// BOITE
	CModel * boite = new CModel();
	boite->OuvrirFichier ("box.ase");

	CVector tailleSol (20.0, 20.0, 1.10);
	NewtCollisionBox colSol (tailleSol);
	
	CVector tailleBoite (1.0, 1.0, 1.0);
	NewtCollisionBox colBoite (tailleBoite);

	CVector positionSol (5.0, 6.0, -6.0); // La position du sol
	NewtBody * bodySol = new NewtBody (colSol, sol, positionSol); // Cration de l'objet bodySyl
	bodySol->RotationX (90); // On effectue une rotation de 90 du sol, pour qu'il soit pos horizontalement

	colSol.NewtReleaseCollision (); // On n'oublie pas de librer la primitive de collision
											  // puisqu'elle n'est plus utilise
	
	CVector positionBoite; // Position des botes
	NewtBody * bodyBox [5]; // On cre cinq pointeurs vers des objets NewtBody
	
	for (int i = 0 ; i < 5 ; ++i)
	{
		positionBoite.SetCoordonnees (-2 * i + 4, 20, -8.0); // On dfinit la position de la boite ni
		bodyBox[i] = new NewtBody (colBoite, boite, positionBoite); // On cre l'objet
		bodyBox[i]->NewtBodySetMassMatrix (40); // On dfinit une masse de 40 kg
		bodyBox[i]->NewtBodySetForceAndTorqueCallback (); // On dfinit un callback pour appliquer les forces
		bodyBox[i]->NewtBodySetTransformCallback (); // On dfinit un callback de transformation
		bodyBox[i]->RotationY (-40); // On effectue une rotation de -40 sur l'axe des Y

		_vBody.push_back (bodyBox[i]); // On l'ajoute  notre vector _vBody
	}
	colBoite.NewtReleaseCollision ();

	const NewtMaterialID * mat1 = new NewtMaterialID ();
	const NewtMaterialID * mat2 = new NewtMaterialID ();
	const NewtMaterialID * mat3 = new NewtMaterialID ();
	const NewtMaterialID * mat4 = new NewtMaterialID ();
	const NewtMaterialID * mat5 = new NewtMaterialID ();

	const NewtMaterialID * matDefault = new NewtMaterialID (1);

	NewtMaterialPair * pair1 = new NewtMaterialPair (mat1, mat1);
	_vMaterials.push_back (pair1);
	NewtMaterialPair * pair2 = new NewtMaterialPair (mat1, mat2);
	_vMaterials.push_back (pair2);
	NewtMaterialPair * pair3 = new NewtMaterialPair (mat1, mat3);
	_vMaterials.push_back (pair3);
	NewtMaterialPair * pair4 = new NewtMaterialPair (mat1, mat4);
	_vMaterials.push_back (pair4);
	NewtMaterialPair * pair5 = new NewtMaterialPair (mat1, mat5);
	_vMaterials.push_back (pair5);
	NewtMaterialPair * pair6 = new NewtMaterialPair (mat2, mat2);
	_vMaterials.push_back (pair6);
	NewtMaterialPair * pair7 = new NewtMaterialPair (mat2, mat3);
	_vMaterials.push_back (pair7);
	NewtMaterialPair * pair8 = new NewtMaterialPair (mat2, mat4);
	_vMaterials.push_back (pair8);
	NewtMaterialPair * pair9 = new NewtMaterialPair (mat2, mat5);
	_vMaterials.push_back (pair9);
	NewtMaterialPair * pair10 = new NewtMaterialPair (mat3, mat3);
	_vMaterials.push_back (pair10);
	NewtMaterialPair * pair11 = new NewtMaterialPair (mat3, mat4);
	_vMaterials.push_back (pair11);
	NewtMaterialPair * pair12 = new NewtMaterialPair (mat4, mat4);
	_vMaterials.push_back (pair12);
	NewtMaterialPair * pair13 = new NewtMaterialPair (mat4, mat5);
	_vMaterials.push_back (pair13);
	NewtMaterialPair * pair14 = new NewtMaterialPair (mat5, mat5);
	_vMaterials.push_back (pair14);

	NewtMaterialPair * pairDefault = new NewtMaterialPair (matDefault, matDefault);
	_vMaterials.push_back (pairDefault);
	
	// 1-1
	pair1->NewtMaterialSetDefaultElasticity (0.3f);

	// 1-2
	pair2->NewtMaterialSetDefaultElasticity (0.9f);

	// 1-3
	pair3->NewtMaterialSetDefaultElasticity (0.05f);

	// 1-4
	// Les collisions entre les objets de matriaux 1 et 4 ne se feront pas
	pair4->NewtMaterialSetDefaultCollidable (0);
	pair4->NewtMaterialSetDefaultElasticity (0.3f);

	// 1-5
	pair5->NewtMaterialSetDefaultElasticity (0.25f);
	
	// 2-2
	pair6->NewtMaterialSetDefaultElasticity (0.6f);
	
	// 2-3
	pair7->NewtMaterialSetDefaultElasticity (0.6f);

	// 2-4
	pair8->NewtMaterialSetDefaultElasticity (0.7f);

	// 2-5
	pair9->NewtMaterialSetDefaultElasticity (0.6f);

	// 3-3
	pair10->NewtMaterialSetDefaultElasticity (0.1f);
	
	// 3-4
	pair11->NewtMaterialSetDefaultElasticity (0.05f);

	// 4-4
	pair12->NewtMaterialSetDefaultElasticity (0.8f);
	
	// 4-5
	pair13->NewtMaterialSetDefaultElasticity (0.1f);

	// 5-5
	pair14->NewtMaterialSetDefaultElasticity (0.05f);

	// Default-Default
	pairDefault->NewtMaterialSetDefaultElasticity (0.05f);

	bodySol->NewtBodySetMaterialGroupID (mat1->GetID());
	bodyBox [0]->NewtBodySetMaterialGroupID (mat2->GetID());
	bodyBox [1]->NewtBodySetMaterialGroupID (mat3->GetID());
	bodyBox [2]->NewtBodySetMaterialGroupID (mat4->GetID());
	bodyBox [3]->NewtBodySetMaterialGroupID (mat5->GetID());
	// On ne spcifie rien pour bodyBox [4], il aura donc le material par dfaut

	delete mat1, delete mat2, delete mat3, delete mat4, delete mat5;

	_vBody.push_back (bodySol); // On ajoute le corps  notre vector _vBody
}

void Affichage::Update ()
{
	// On met  jour tous les objets en mettant  jour le monde Newton
	_nWorld->NewtUpdate ((float)1.0f/60.0f);

	//systeme->update();
}

void Affichage::Render ()
{
	glClearColor(0.0f, 0.0f, 0.3f, 0.0f);
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity ();

	gluLookAt (0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
	glRotatef(15, 1, 0, 0);
   glRotatef(40, 0, 1, 0);
	
	// On active le mode debug
	if (_debug)
		_nWorld->DebugShowCollision ();
	
	// On dessine tous les objets
	std::for_each (_vBody.begin(), _vBody.end(), std::mem_fun(&NewtBody::DrawModel));

	SDL_GL_SwapBuffers ();
}

void Affichage::Shutdown ()
{
	// On libre la mmoire des vector
	std::for_each (_vBody.begin(), _vBody.end(), Delete());
	_vBody.clear();
	
	std::for_each (_vMaterials.begin(), _vMaterials.end(), Delete());
	_vMaterials.clear();
	
	_nWorld->Destroy (); // On libre la mmoire alloue par Newton
}

