#ifndef MESH_HPP
#define MESH_HPP

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <algorithm>
#include <corona.h>
#include "Vector.hpp"

using namespace std;

struct aseVertex
{
   float x, y, z;
};

struct aseNormale
{
   float x, y, z;
};

struct aseTexture
{
   float u, v;
};

struct aseMaterial
{
   CVector ambDif; // Valeurs ambiant & diffuse
   CVector specular; // Valeurs specular
   std::string nomTexture; // Nom de la texture
};

struct aseFace
{
   int indexVertices [3];
   int indexTextures [3];
};

class CMesh
{
   public:
      CMesh ();
      ~CMesh ();

      bool OuvrirString (std::string contenuObj);
      bool OuvrirTexture ();
      void AssignerMaterial (aseMaterial sMaterial);
      void Shutdown ();

      void CreerTableauVertices ();
      void CreerTableauTexture ();
      void Dessiner ();

      int GetNumPoints () const {return m_iNumPoints;}
      int GetNumVertices () const {return m_iNumVertices;}
      int GetNumFaces () const {return m_iNumFaces;}

      std::vector <aseFace> GetFaces () const {return m_vFaces;}
      std::vector <aseVertex> GetVertices () const {return m_vVertices;}

   private:
      int m_iNumPoints;
      int m_iNumVertices;
      int m_iNumFaces;
      int m_iNumTCoords;

      bool m_bTexture;
      unsigned int m_uiTexture;

      unsigned int m_uiVBO;

      std::vector <aseVertex> m_vVertices;
      std::vector <aseNormale> m_vNormales;
      std::vector <aseFace> m_vFaces;
      std::vector <aseTexture> m_vTexCoords;

      aseMaterial m_sMaterial;

      float * tabVertices;
      float * tabTextures;
};

#endif // MESH_HPP
