#include "Model.hpp"

CModel::CModel()
   : m_iNumMesh (0), m_iNumMaterials (0), _matrix (), _offset ()
{
}

CModel::~CModel()
{
   Shutdown ();
}

void CModel::Shutdown ()
{
	//std::for_each (m_vMeshs.begin(), m_vMeshs.end(), Delete());

   m_vMeshs.clear (); // On vide le vector
   m_vMaterials.clear ();
}

bool CModel::OuvrirFichier (std::string nomFichier)
{
   ifstream fichier (nomFichier.c_str(), std::ios::in); // On ouvre le fichier

   if (fichier.fail())
   {
      std::cerr << "Impossible d'ouvrir le fichier " << nomFichier;
      return false;
   }

   // On commence pour nettoyer le contenu de l'objet, au cas ou,...
   Shutdown ();

   // On met le contenu du fichier dans un buffer
   std::stringstream buffer;
   std::string strBuffer;
   buffer << fichier.rdbuf();
   strBuffer = buffer.str ();
   fichier.close (); // On ferme le fichier, on en a plus besoin
   std::string tmp; // Chane temporaire

   // Plusieurs structures et valeurs temporaires
   std::string contenuObj;

   // On rcupre plusieurs positions
   while (buffer >> tmp)
   {
      if (tmp == "*MATERIAL")
      {
         buffer >> tmp;

         aseMaterial matTmp;

         while (tmp != "}") // Fermeture de la partie Material
         {
            if (tmp == "*MATERIAL_DIFFUSE")
               buffer >> matTmp.ambDif.x >> matTmp.ambDif.y >> matTmp.ambDif.z;
            else if (tmp == "*MATERIAL_SPECULAR")
               buffer >> matTmp.specular.x >> matTmp.specular.y >> matTmp.specular.z;
            else if (tmp == "*BITMAP")
               buffer >> matTmp.nomTexture;

            buffer >> tmp; // On va au mot suivant
         }

         m_vMaterials.push_back (matTmp);
      }
      else if (tmp == "*MESH")
      {
         int numMat;

         buffer >> tmp;

         while (tmp != "*MATERIAL_REF") // Fermeture de la gometrie
         {
            contenuObj += tmp;
            contenuObj += ' ';

            buffer >> tmp;
         }

         buffer >> numMat; // On enregistre le numro du materiel

         CMesh * tmpObj = new CMesh ();
         tmpObj->AssignerMaterial (m_vMaterials [numMat]);
         if (!tmpObj->OuvrirString (contenuObj))
         {
            std::cerr << "Impossible d'ouvrir le mesh n " << m_iNumMesh;
            return false;
         }
         m_vMeshs.push_back (tmpObj);

         contenuObj.clear ();
         ++m_iNumMesh;
      }
   }
	
	SetOffset ();

   return true;
}

void CModel::SetOffset ()
{
	_offset = GetLongueurs ();
	_offset /= 2.0;
}

CVector CModel::GetLongueurs () const
{
	CVector min, max; // Deux vecteurs reprsentant la valeur la plus petite et la plus
							// grande
	
	for (int i = 0 ; i < GetNumMeshs() ; ++i)
	{
		std::vector <aseVertex> vertexTmp = m_vMeshs [i]->GetVertices ();

		if (i == 0) // Premire itration
		{
			min.SetCoordonnees (vertexTmp [0].x, vertexTmp [0].y, vertexTmp [0].z);
			max.SetCoordonnees (vertexTmp [0].x, vertexTmp [0].y, vertexTmp [0].z);
		}

		for (int j = 0 ; j < m_vMeshs [i]->GetNumVertices () ; ++j)
		{
			// Pour les x
			if (vertexTmp [j].x < min.x)
				min.x = vertexTmp [j].x;
			else if (vertexTmp [j].x > max.x)
				max.x = vertexTmp [j].x;

			// Pour les y
			if (vertexTmp [j].y < min.y)
				min.y = vertexTmp [j].y;
			else if (vertexTmp [j].y > max.y)
				max.y = vertexTmp [j].y;

			// Pour les z
			if (vertexTmp [j].z < min.z)
				min.z = vertexTmp [j].z;
			else if (vertexTmp [j].z > max.z)
				max.z = vertexTmp [j].z;
		}
	}
	
	//CVector final ((abs (min.x) + abs (max.x)), (abs (min.y) + abs (max.y)), (abs (min.z) + abs (max.z)));
	CVector final ((float)(max.x - min.x), (float)(max.y - min.y), (float)(max.z - min.z));
	
	std::cout << final.x << ' ' << final.y << ' ' << final.z;
	return final;
}

void CModel::SetPosition (const float * mat)
{
	for (int i = 0 ; i < 16 ; ++i)
		_matrix._matrix [i] = mat [i];
}

void CModel::Dessiner ()
{
   glPushMatrix (); // On sauvegarde la matrice actuelle
      glMultMatrixf (&_matrix._matrix [0]);
	
   for (int i = 0 ; i < m_iNumMesh ; ++i)
      m_vMeshs [i]->Dessiner ();

	glPopMatrix ();
}
