/// Classe NewtBody. Cette classe englobe plusieurs fonctions relatives  la gestion
/// des corps dans Newton Game Dynamics. Toutes les fonctions ne sont videmment pas
/// encore intgres, cela viendra avec le temps :)

#ifndef NEWTBODY_HPP
#define NEWTBODY_HPP

#include "NewtWorld.hpp"
#include "NewtCollision.hpp"

#include "Model.hpp"

class NewtBody
{
	public:
		// Constructeur / Destructeur
		NewtBody (const NewtCollision & nCollision, CModel * model, const CVector & position);
		~NewtBody ();

		inline CModel & GetModel ();
		inline NewtonBody * GetBody () const;

		// Effectue une rotation du corps sur l'axe X
		inline void RotationX (const int rotationX);
		inline void RotationY (const int rotationY);
		inline void RotationZ (const int rotationZ);

		// Fonction qui dessinera le modle
		void DrawModel ();

		/// --------------------------------------------------------------------------
		/// FONCTIONS RELATIVES A NEWTON GAME DYNAMICS
		/// --------------------------------------------------------------------------
		// Dtruit le corps
		void NewtDestroyBody () const;
		
		// Assigne la masse et des valeurs d'inertie
		void NewtBodySetMassMatrix (const float masse);
		
		// Permet de dfinir un nouveau centre de masse (ou centre d'inertie)
		void NewtBodySetCentreOfMass ();

		// Applique le "continuous collision mode" au corps
		void NewtBodySetContinuousCollisionMode (unsigned int etat) const;

		// Applique un callback au corps
		void NewtBodySetForceAndTorqueCallback () const;
		
		// Applique un callback de transformation au corps
		void NewtBodySetTransformCallback () const; 
		
		// Applique un matrial au corps
		void NewtBodySetMaterialGroupID (const int matID) const;

		// Renvoie l'identifiant du matriel
		int NewtBodyGetMaterialGroupID () const;

	private:
		// On pointeur constant vers un objet NewtWorld
		const NewtWorld * _nWorld;
		
		// On objet NewtCollision
		NewtCollision _nCollision;
		
		// Un pointeur vers une structure NewtonBody
		NewtonBody * _nBody;
		
		// Masse de l'objet
		float _masse;
		
		// Le modle 3D
		CModel _model;
};

CModel & NewtBody::GetModel ()
{
	return _model;
}

NewtonBody * NewtBody::GetBody () const
{
	return _nBody;
}

void NewtBody::RotationX (const int rotationX)
{
	Matrix matrixBody; // La matrice du corps

	Matrix matrixRotation; // La matrice  laquelle on va effectuer les rotations
	matrixRotation.LoadIdentity (); // On charge la matrice identit
	matrixRotation.RotationX (rotationX); // On effectue la rotation

	// On rcupre la matrice de transformation du corps
	NewtonBodyGetMatrix (_nBody, &matrixBody._matrix [0]);

	matrixRotation *= matrixBody; // On multiplie les deux matrices
	
	// Puis on passe la nouvelle matrice  la fonction NewtonBodySetMatrix
	NewtonBodySetMatrix (_nBody, &matrixRotation._matrix [0]);

	_model.SetPosition (&matrixRotation._matrix [0]);
}

void NewtBody::RotationY (const int rotationY)
{
	Matrix matrixBody; // La matrice du corps

	Matrix matrixRotation; // La matrice  laquelle on va effectuer les rotations
	matrixRotation.LoadIdentity (); // On charge la matrice identit
	matrixRotation.RotationY (rotationY); // On effectue la rotation

	// On rcupre la matrice de transformation du corps
	NewtonBodyGetMatrix (_nBody, &matrixBody._matrix [0]);

	matrixRotation *= matrixBody; // On multiplie les deux matrices
	
	// Puis on passe la nouvelle matrice  la fonction NewtonBodySetMatrix
	NewtonBodySetMatrix (_nBody, &matrixRotation._matrix [0]);

	_model.SetPosition (&matrixRotation._matrix [0]);
}

void NewtBody::RotationZ (const int rotationZ)
{
	Matrix matrixBody; // La matrice du corps

	Matrix matrixRotation; // La matrice  laquelle on va effectuer les rotations
	matrixRotation.LoadIdentity (); // On charge la matrice identit
	matrixRotation.RotationZ (rotationZ); // On effectue la rotation

	// On rcupre la matrice de transformation du corps
	NewtonBodyGetMatrix (_nBody, &matrixBody._matrix [0]);

	matrixRotation *= matrixBody; // On multiplie les deux matrices
	
	// Puis on passe la nouvelle matrice  la fonction NewtonBodySetMatrix
	NewtonBodySetMatrix (_nBody, &matrixRotation._matrix [0]);

	_model.SetPosition (&matrixRotation._matrix [0]);
}

#endif // NEWTBODY_HPP
