/// Classe NewtCollision. Cette classe est une classe de base pour toutes les autres
/// primitives de collision. Elle regroupe pas mal de fonction relatives  la gestion
/// des primitives de collision.

#ifndef NEWTCOLLISION_HPP
#define NEWTCOLLISION_HPP

#include "NewtWorld.hpp"

class NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollision ();
		virtual ~NewtCollision ();

		// Renvoi la primitive de collision
		inline NewtonCollision * GetCollision () const;

		/// --------------------------------------------------------------------------
		/// FONCTIONS RELATIVES A NEWTON GAME DYNAMICS
		/// --------------------------------------------------------------------------
		// Permet de librer la mmoire. La fonction doit absolument tre appele une fois
		// la primitive de collision attribue  un corps
		virtual void NewtReleaseCollision () const;

		// Calcule une approximation du volume de la primitive de collision
		virtual float NewtConvexCollisionCalculateVolume () const;

		// Calcule le centre de masse (ou centre d'inertie) de la primitive de collision
		virtual void NewtConvexCollisionCalculateInertialMatrix (CVector & inertie, CVector & origine);
		
		// Permet de rendre une primitive de collision unique
		virtual void NewtCollisionMakeUnique () const;

	protected:
		// Pointeur vers un objet NewtWorld
		NewtWorld * _nWorld;
		
		// Pointeur vers une structure NewtonCollision
		NewtonCollision * _nCollision;
};

NewtonCollision * NewtCollision::GetCollision () const
{
	return _nCollision;
}

#endif // NEWTCOLLISION_HPP
