/// Fichier d'en-tte NewtCollisionPrimitives, qui regroupe plusieurs classes pour
/// chaque primitive de collision (sauf les arbres de collision). Chacune de ces classes
/// drive de NewtCollision et bnficie donc de toutes ses mthodes membres.

#ifndef NEWTCOLLISIONPRIMITIVES_HPP
#define NEWTCOLLISIONPRIMITIVES_HPP

#include <algorithm>
#include <functional>

#include "NewtCollision.hpp"
#include "Model.hpp"
#include "Matrix.hpp"


/// Classe pour la primitive de collision invisible
class NewtCollisionNull : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionNull ();
		~NewtCollisionNull ();
};

/// Classe pour les botes
class NewtCollisionBox : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionBox (const CVector & taille);
		~NewtCollisionBox ();
};


/// Classe pour les sphres
class NewtCollisionSphere : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionSphere (const CVector & taille);
		~NewtCollisionSphere ();
};


/// Classe pour les cnes
class NewtCollisionCone : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionCone (const float rayon, const float hauteur);
		~NewtCollisionCone ();
};


/// Classe pour les capsules
class NewtCollisionCapsule : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionCapsule (const float rayon, const float hauteur);
		~NewtCollisionCapsule ();
};


/// Classe pour les cylindres
class NewtCollisionCylinder : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionCylinder (const float rayon, const float hauteur);
      ~NewtCollisionCylinder ();
};


/// Classe pour les cylindres chanfreins
class NewtCollisionChamferCylinder : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionChamferCylinder (const float rayon, const float hauteur);
		~NewtCollisionChamferCylinder ();
};


/// Classe pour les enveloppes convexes
class NewtCollisionConvexHull : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionConvexHull (CModel * model);
		~NewtCollisionConvexHull ();
};

/// Classe pour les tableau d'enveloppes convexes
class NewtCollisionCompound : public NewtCollision
{
	public:
		// Constructeur / Destructeur
		NewtCollisionCompound (CModel * model);
		~NewtCollisionCompound ();
};

#endif // NEWTCOLLISIONPRIMITIVES_HPP
