#include "NewtMaterialPair.hpp"

NewtMaterialPair::NewtMaterialPair (const NewtMaterialID * mat1, const NewtMaterialID * mat2)
	: _mat1 (mat1), _mat2 (mat2)
{
	_nWorld = NewtWorld::GetInstance ();
}

NewtMaterialPair::~NewtMaterialPair ()
{
}

// Dfinit si deux matriaux peuvent "entrer en collision". Si le couple de matriaux ont
// comme valeur 0 (non collidable), alors les corps ayant ces matriaux ne rentreront pas
// en contact et se "transperserons"
void NewtMaterialPair::NewtMaterialSetDefaultCollidable (int etat)
{
	if (etat < 0 || etat > 1)
		etat = 1;

	NewtonMaterialSetDefaultCollidable (_nWorld->GetNewtonWorld(), _mat1->GetID(),
												   _mat2->GetID(), etat);
} 

// Dfinit un degr de "douceur" du corps. A vrai dire, je n'ai pas vu beaucoup de diffrence
// en changeant ce paramtre. J'en ferai d'autres plus tard. Cette valeur doit tre
// comprise, de prfrence, entre 0.0 et 1.0.
void NewtMaterialPair::NewtMaterialSetDefaultSoftness (const float softnessValue) const
{
	NewtonMaterialSetDefaultSoftness (_nWorld->GetNewtonWorld(), _mat1->GetID(),
												 _mat2->GetID(), softnessValue);
}

// Dfinit un degr d'elasticit du matriel. Plus cette valeur sera haute, plus le corps
// ayant ce matriau "rebondira", comme une balle rebondissante. Cette valeur doit tre
// comprise, de prfrence, entre 0.0 et 1.0.
void NewtMaterialPair::NewtMaterialSetDefaultElasticity (const float elasticityValue) const
{
	NewtonMaterialSetDefaultElasticity (_nWorld->GetNewtonWorld(), _mat1->GetID(),
												   _mat2->GetID(), elasticityValue);
}

// Dfinit les valeurs de friction.
void NewtMaterialPair::NewtMaterialSetDefaultFriction (const float staticFriction, 
																		 const float kineticFriction) const
{
	NewtonMaterialSetDefaultFriction (_nWorld->GetNewtonWorld(), _mat1->GetID(),
												 _mat2->GetID(), staticFriction, kineticFriction);
}

