/// Classe NewtMaterialPair. Reprsente une paire de deux matriaux

#ifndef NEWTMATERIALPAIR_HPP
#define NEWTMATERIALPAIR_HPP

#include <iostream>
#include <string>

#include "NewtMaterialID.hpp"

class NewtMaterialPair
{
	public:
		// Constructeur / Destructeur
		NewtMaterialPair (const NewtMaterialID * mat1, const NewtMaterialID * mat2);
		~NewtMaterialPair ();

		/// --------------------------------------------------------------------------
		/// FONCTIONS RELATIVES A NEWTON GAME DYNAMICS
		/// --------------------------------------------------------------------------
		// Dfinit si deux matriaux rentrent ou non en contact
		void NewtMaterialSetDefaultCollidable (int etat);

		// Dfinit la "douceur" du matriel
		void NewtMaterialSetDefaultSoftness (const float softnessValue) const;

		// Dfinit l'elasticit du matriel
		void NewtMaterialSetDefaultElasticity (const float elasticityValue) const;

		// Dfinit les valeurs de friction du matriel
		void NewtMaterialSetDefaultFriction (const float staticFriction, const float kineticFriction) const;

	private:
		const NewtWorld * _nWorld; // Pointeur vers un objet NewtWorld

		const NewtMaterialID * _mat1; // Pointeur vers le matriel n1
		const NewtMaterialID * _mat2; // Pointeur vers le matriel n2
};

#endif // NEWTMATERIALPAIR_HPP
