#include "Mesh.hpp"

CMesh::CMesh ()
   : m_iNumVertices (0), m_iNumFaces (0), m_iNumTCoords (0), tabVertices (NULL),
     tabTextures (NULL)
{
   m_bTexture = false; // Pas de texture de base
   m_iNumPoints = 3; // On ne gre pas les quads, seulement les triangles
}

CMesh::~CMesh ()
{
   Shutdown ();
}

void CMesh::Shutdown ()
{
   // On vide tous les vectors
   m_vVertices.clear ();
   m_vNormales.clear ();
   m_vFaces.clear ();
   m_vTexCoords.clear ();

   glDeleteTextures (1, &m_uiTexture);
   m_bTexture = false;

   glDeleteBuffers (1, &m_uiVBO);
}

bool CMesh::OuvrirString (std::string contenuObj)
{
   std::string chaine;
   std::istringstream iss (contenuObj);

   int iCurrentTex = 0;

   while (iss >> chaine)
   {
      if (chaine == "*MESH_NUMVERTEX") // *MESH_NUMVERTEX nbVertex
      {
         iss >> m_iNumVertices;
      }

      else if (chaine == "*MESH_NUMFACES") // *MESH_NUMFACES nbFaces
      {
         iss >> m_iNumFaces;
      }

      else if (chaine == "*MESH_NUMTVERTEX") // *MESH_NUMTVERTEX nbTextureCoordonees
      {
         iss >> m_iNumTCoords;

         m_bTexture = true; // Il y a des textures
      }
   }

   iss.clear(); // On remet  zro l'EOF pour la prochaine fois
   iss.seekg (0); // Et on replace le curseur au dbut de celui-ci


   // On alloue de la mmoire pour les vector, suivant les valeurs trouves plus haut
   m_vVertices.reserve (m_iNumVertices);
   m_vFaces.reserve (m_iNumFaces);
   m_vNormales.reserve (m_iNumFaces * m_iNumPoints); // Le nombre de faces * nbPoints
   m_vTexCoords.reserve (m_iNumTCoords);

   // A prsent que la mmoire est rserv, nous allons parcourir une nouvelle fois
   // le buffer mais en rcuprant toutes les donnes relatives aux vertices, normales,
   // et faces,...

   while (iss >> chaine)
   {
      if (chaine == "*MESH_VERTEX")
      {
         aseVertex tmpStruct; // Structure temporaire
         int tmpInt;

         iss >> tmpInt >> tmpStruct.x >> tmpStruct.y >> tmpStruct.z;

         m_vVertices.push_back (tmpStruct); // On ajoute la structure
      }

      else if (chaine == "*MESH_FACE")
      {
         aseFace tmpStruct;

         // La chaine rcupre se prsentera comme ceci :
         // 0: A: 0 B: 1 C: 2 AB: 1 BC: 1
         std::string tmpStr;

         iss >> tmpStr >> tmpStr >> tmpStruct.indexVertices [0] >> tmpStr >>
                tmpStruct.indexVertices [1] >> tmpStr >> tmpStruct.indexVertices [2];

         m_vFaces.push_back (tmpStruct); // On remplit le vecteur
      }

      else if (chaine == "*MESH_TVERT")
      {
         aseTexture tmpStruct;
         int tmpInt;

         iss >> tmpInt >> tmpStruct.u >> tmpStruct.v;

         m_vTexCoords.push_back (tmpStruct);
      }

      else if (chaine == "*MESH_TFACE")
      {
         std::string tmpStr;
         int indice;

         iss >> indice;

         iss >> m_vFaces [iCurrentTex].indexTextures [0] >>
                m_vFaces [iCurrentTex].indexTextures [1] >>
                m_vFaces [iCurrentTex].indexTextures [2];

         ++iCurrentTex;
      }

      else if (chaine == "*MESH_VERTEXNORMAL")
      {
         aseNormale tmpStruct;
         std::string tmpStr;

         iss >> tmpStr >> tmpStruct.x >> tmpStruct.y >> tmpStruct.z;

         m_vNormales.push_back (tmpStruct);
      }
   }

   CreerTableauVertices (); // On duplique les vertices

   // Si le mesh a une texture, on effectue certaines manipulations diffrentes
   if (m_bTexture)
   {
      CreerTableauTexture ();

      int taille = sizeof (float) * (m_iNumFaces * 9);
      int taille2 = sizeof (float) * (m_iNumFaces * 6);
      int sizeBuffer = 0;

      glGenBuffers (1, &m_uiVBO);
      glBindBuffer (GL_ARRAY_BUFFER, m_uiVBO);
      glBufferData (GL_ARRAY_BUFFER, taille * 2 + taille2, NULL, GL_STATIC_DRAW);
      glBufferSubData (GL_ARRAY_BUFFER, 0, taille, tabVertices);
      glBufferSubData (GL_ARRAY_BUFFER, taille, taille2, tabTextures);
      glBufferSubData (GL_ARRAY_BUFFER, taille + taille2, taille, &m_vNormales [0].x);

      glGetBufferParameteriv (GL_ARRAY_BUFFER, GL_BUFFER_SIZE, &sizeBuffer);

      // On vrifie que la mmoire ait bien t alloue
      if (sizeBuffer <= 0)
      {
         std::cerr << "Impossible d'allouer de la mmoire pour les VBOs.";
         return false;
      }

      // A prsent que les donnes sont dans la mmoire du GPU, on peut les supprimer
      delete [] tabVertices, tabVertices = NULL;
      delete [] tabTextures, tabTextures = NULL;
   }
   else
   {
      int taille = sizeof (float) * (m_iNumFaces * 9);
      int sizeBuffer = 0;

      glGenBuffers (1, &m_uiVBO);
      glBindBuffer (GL_ARRAY_BUFFER, m_uiVBO);
      glBufferData (GL_ARRAY_BUFFER, taille * 2, NULL, GL_STATIC_DRAW);
      glBufferSubData (GL_ARRAY_BUFFER, 0, taille, tabVertices);
      glBufferSubData (GL_ARRAY_BUFFER, taille, taille, &m_vNormales [0].x);

      glGetBufferParameteriv (GL_ARRAY_BUFFER, GL_BUFFER_SIZE, &sizeBuffer);

      // On vrifie que la mmoire ait bien t alloue
      if (sizeBuffer <= 0)
      {
         std::cerr << "Impossible d'allouer de la mmoire pour les VBOs.";
         return false;
      }

      // A prsent que les donnes sont dans la mmoire du GPU, on peut les supprimer
      delete [] tabVertices, tabVertices = NULL;
   }

   return true;
}

void CMesh::AssignerMaterial (aseMaterial sMaterial)
{
   m_sMaterial.ambDif = sMaterial.ambDif;
   m_sMaterial.specular = sMaterial.specular;

   if (!sMaterial.nomTexture.empty() && (m_sMaterial.nomTexture != "\"Untitled\""))
   {
      m_sMaterial.nomTexture = sMaterial.nomTexture;

      m_sMaterial.nomTexture.erase (0, 1);
      m_sMaterial.nomTexture.erase ((m_sMaterial.nomTexture.size() - 1), 1);

      OuvrirTexture ();
      m_bTexture = true;
   }
}

void CMesh::CreerTableauVertices ()
{
   // On duplique toutes les vertices pour pouvoir utiliser les VBO
   tabVertices = new float [m_iNumFaces * 9];

   for (int i = 0 ; i < m_iNumFaces ; ++i)
   {
      tabVertices [9 * i] = m_vVertices [m_vFaces [i].indexVertices [0]].x;
      tabVertices [9 * i + 1] = m_vVertices [m_vFaces [i].indexVertices [0]].y;
      tabVertices [9 * i + 2] = m_vVertices [m_vFaces [i].indexVertices [0]].z;

      tabVertices [9 * i + 3] = m_vVertices [m_vFaces [i].indexVertices [1]].x;
      tabVertices [9 * i + 4] = m_vVertices [m_vFaces [i].indexVertices [1]].y;
      tabVertices [9 * i + 5] = m_vVertices [m_vFaces [i].indexVertices [1]].z;

      tabVertices [9 * i + 6] = m_vVertices [m_vFaces [i].indexVertices [2]].x;
      tabVertices [9 * i + 7] = m_vVertices [m_vFaces [i].indexVertices [2]].y;
      tabVertices [9 * i + 8] = m_vVertices [m_vFaces [i].indexVertices [2]].z;
   }
}

void CMesh::CreerTableauTexture ()
{
   tabTextures = new float [m_iNumFaces * 6];

   for (int i = 0 ; i < m_iNumFaces ; ++i)
   {
      tabTextures [6 * i] = m_vTexCoords [m_vFaces [i].indexTextures [0]].u;
      tabTextures [6 * i + 1] = m_vTexCoords [m_vFaces [i].indexTextures [0]].v;

      tabTextures [6 * i + 2] = m_vTexCoords [m_vFaces [i].indexTextures [1]].u;
      tabTextures [6 * i + 3] = m_vTexCoords [m_vFaces [i].indexTextures [1]].v;

      tabTextures [6 * i + 4] = m_vTexCoords [m_vFaces [i].indexTextures [2]].u;
      tabTextures [6 * i + 5] = m_vTexCoords [m_vFaces [i].indexTextures [2]].v;
   }
}

bool CMesh::OuvrirTexture ()
{
   corona::Image * image = corona::OpenImage (m_sMaterial.nomTexture.c_str());

   if (!image)
      std::cerr << "Impossible d'ouvrir " << m_sMaterial.nomTexture << ".";

   FlipImage (image, corona::CA_X);

   int width = image->getWidth();
   int height = image->getHeight();
   void * pixels = image->getPixels();
   corona::PixelFormat format = image->getFormat();

   glGenTextures (1, &m_uiTexture);
   glBindTexture (GL_TEXTURE_2D, m_uiTexture);

   glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

   switch (format)
   {
      case corona::PF_R8G8B8A8:
         gluBuild2DMipmaps (GL_TEXTURE_2D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
         break;
      case corona::PF_R8G8B8:
         gluBuild2DMipmaps (GL_TEXTURE_2D, GL_RGB, width, height, GL_RGB, GL_UNSIGNED_BYTE, pixels);
         break;
      default:
         std::cerr << "Impossible de traiter l'image (ni RGB, ni RGBA)";
         return false;
         break;
   }

   delete image;

   return true;
}

void CMesh::Dessiner ()
{
   glColorMaterial (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
   glColor3f (m_sMaterial.ambDif.x, m_sMaterial.ambDif.y, m_sMaterial.ambDif.z);
   glColorMaterial (GL_FRONT_AND_BACK, GL_SPECULAR);
   glColor3f (m_sMaterial.specular.x, m_sMaterial.specular.y, m_sMaterial.specular.z);
   glColorMaterial (GL_FRONT_AND_BACK, GL_EMISSION);
   glColor3f (0.0, 0.0, 0.0);
   glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 100);

   glEnable (GL_COLOR_MATERIAL);

   int taille = sizeof (float) * (m_iNumFaces * 9);
   int taille2 = sizeof (float) * (m_iNumFaces * 6);

   if (m_bTexture)
   {
      glEnable (GL_TEXTURE_2D);
      glBindTexture (GL_TEXTURE_2D, m_uiTexture);

      glEnableClientState (GL_VERTEX_ARRAY);
      glEnableClientState (GL_NORMAL_ARRAY);
      glEnableClientState (GL_TEXTURE_COORD_ARRAY);

      glBindBuffer (GL_ARRAY_BUFFER, m_uiVBO);

      glVertexPointer (3, GL_FLOAT, 0, (void *) NULL);
      glTexCoordPointer (2, GL_FLOAT, 0, (void *) (taille));
      glNormalPointer (GL_FLOAT, 0, (void *) (taille + taille2));
      glDrawArrays (GL_TRIANGLES, 0, m_iNumFaces * 3);

      glBindBuffer (GL_ARRAY_BUFFER, 0);

      glDisableClientState (GL_TEXTURE_COORD_ARRAY);
      glDisableClientState (GL_NORMAL_ARRAY);
      glDisableClientState (GL_VERTEX_ARRAY);

      glDisable (GL_TEXTURE_2D);
   }
   else
   {
      glEnableClientState (GL_VERTEX_ARRAY);
      glEnableClientState (GL_NORMAL_ARRAY);

      glBindBuffer (GL_ARRAY_BUFFER, m_uiVBO);

      glVertexPointer (3, GL_FLOAT, 0, (void *) NULL);
      glNormalPointer (GL_FLOAT, 0, (void *) (taille));
      glDrawArrays (GL_TRIANGLES, 0, m_iNumFaces * 3);

      glBindBuffer (GL_ARRAY_BUFFER, 0);

      glDisableClientState (GL_NORMAL_ARRAY);
      glDisableClientState (GL_VERTEX_ARRAY);
   }
}

