#include "NewtCollisionPrimitives.hpp"

/// -------------------------- NULL ---------------------------------
// Constructeur, qui cre la primitive de collision. Je n'ai jamais experiment cette
// primitive de collision invisible pour le moment, je vais juste vous recopier la doc.
// D'aprs elle, cette primitive de collision est utile lorsque l'on a besoin de crer
// des corps qui ne vont jamais entrer en collision avec d'autres corps, par exemple
// le cou d'un ragdoll, ou la partie interne d'une structure articule. Voil, d'aprs
// ce que j'y comprends, et pour reprendre l'exemple du cou, a doit tre utile avec 
// les joints (on fait la tte, on fait le ventre, et on fait une primitive invisible
// comme celle-ci, et qui sera reli par des joints  la tte et au corps).
NewtCollisionNull::NewtCollisionNull ()
	: NewtCollision ()
{
	_nCollision = NewtonCreateNull (_nWorld->GetNewtonWorld());
}

// Destructeur
NewtCollisionNull::~NewtCollisionNull ()
{
}


/// -------------------------- BOITES ---------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionBox::NewtCollisionBox (const CVector & taille)
	: NewtCollision ()
{
	_nCollision = NewtonCreateBox (_nWorld->GetNewtonWorld(), taille.x, taille.y, taille.z, NULL);
}

// Destructeur
NewtCollisionBox::~NewtCollisionBox ()
{
}


/// --------------------------- SPHERES -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionSphere::NewtCollisionSphere (const CVector & taille)
	: NewtCollision ()
{
	_nCollision = NewtonCreateSphere (_nWorld->GetNewtonWorld(), taille.x, taille.y, taille.z, NULL);
}

// Destructeur
NewtCollisionSphere::~NewtCollisionSphere ()
{
}


/// --------------------------- CONE -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionCone::NewtCollisionCone (const float rayon, const float hauteur)
	: NewtCollision ()
{
	_nCollision = NewtonCreateCone (_nWorld->GetNewtonWorld(), rayon, hauteur, NULL);
}

// Destructeur
NewtCollisionCone::~NewtCollisionCone ()
{
}


/// --------------------------- CAPSULE -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionCapsule::NewtCollisionCapsule (const float rayon, const float hauteur)
	: NewtCollision ()
{
	_nCollision = NewtonCreateCapsule (_nWorld->GetNewtonWorld(), rayon, hauteur, NULL);
}

// Destructeur
NewtCollisionCapsule::~NewtCollisionCapsule ()
{
}


/// --------------------------- CYLINDRE -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionCylinder::NewtCollisionCylinder (const float rayon, const float hauteur)
	: NewtCollision ()
{
	_nCollision = NewtonCreateCylinder (_nWorld->GetNewtonWorld(), rayon, hauteur, NULL);
}

// Destructeur
NewtCollisionCylinder::~NewtCollisionCylinder ()
{
}


/// --------------------------- CYLINDRES CHANFREINES -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionChamferCylinder::NewtCollisionChamferCylinder (const float rayon, const float hauteur)
	: NewtCollision ()
{
	_nCollision = NewtonCreateChamferCylinder (_nWorld->GetNewtonWorld(), rayon, hauteur, NULL);
}

// Destructeur
NewtCollisionChamferCylinder::~NewtCollisionChamferCylinder ()
{
}


/// --------------------------- ENVELOPPES CONVEXES -------------------------------
// Constructeur, qui cre la primitive de collision
NewtCollisionConvexHull::NewtCollisionConvexHull (CModel * model)
	: NewtCollision ()
{
	// Rappelons que les enveloppes convexes (contrairement aux tableaux d'enveloppes
	// convexes - ou compound object -), n'acceptent les vertices que d'un seul mesh,
	// c'est pourquoi on rcupre les vertices du premier mesh seulement. Si le modle
	// est compos de plusieurs meshs, il faudra utiliser les tableaux d'enveloppes
	// convexes
	_nCollision = NewtonCreateConvexHull (_nWorld->GetNewtonWorld(), model->GetMesh(0)->GetNumVertices(), 
													  &model->GetMesh(0)->GetVertices()[0].x, sizeof (float) * 3, NULL);
}

// Destructeur
NewtCollisionConvexHull::~NewtCollisionConvexHull ()
{
}


/// --------------------------- TABLEAU D'ENVELOPPES CONVEXES ------------------------
// Constructeur, qui cre la primitive de collision. Attention, les tableaux d'enveloppes
// convexes (ou objet compound), ne PEUVENT PAS tre partags par plusieurs objets, contrairement
//  toutes les autres primitives de collision
NewtCollisionCompound::NewtCollisionCompound (CModel * model)
	: NewtCollision ()
{
	// Pour stocker chaque enveloppe convexe dans une primitive de collision finale
   // les englobant toutes (notre variable _nCollision), on cre un vector compos
   // de plusieurs primitives de collision
   std::vector <NewtonCollision *> enveloppesConvexes;

   // On parcourt chaque mesh du modle.
   for (int i = 0 ; i < model->GetNumMeshs () ; ++i)
   {
		// Puis on cre notre primitive de collision avec les coordonnes passes  la
      // fonction NewtonCreateConvexHull
		_nCollision = NewtonCreateConvexHull (_nWorld->GetNewtonWorld(), model->GetMesh(i)->GetNumVertices(),
														  &model->GetMesh(i)->GetVertices()[0].x, sizeof (float) * 3, NULL);

      // On remplit ensuite notre vector de NewtonCollision avec la primitive de collision
      // prcdemment remplie
      enveloppesConvexes.push_back (_nCollision);
   }

   // Puis on attribue  notre variable nCollision la primitive de collision finale,
   // constitue de plusieurs enveloppes convexes
	_nCollision = NewtonCreateCompoundCollision (_nWorld->GetNewtonWorld(), model->GetNumMeshs (), &enveloppesConvexes[0]);

	 // On libre chaque primitive de collision contenue dans le vector
   std::for_each (enveloppesConvexes.begin (), enveloppesConvexes.end (), bind1st(ptr_fun(&NewtonReleaseCollision), _nWorld->GetNewtonWorld()));
	enveloppesConvexes.clear();
}

// Destructeur
NewtCollisionCompound::~NewtCollisionCompound ()
{
}